package com.donglan.BYWeApp.config;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.api.impl.WxMaServiceImpl;
import cn.binarywang.wx.miniapp.config.impl.WxMaDefaultConfigImpl;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.util.Map;

/**
 * @author shj
 * @date 2021/5/11 2:21 下午
 */
@Data
@Configuration
public class WxMaConfig {
    /**
     * 设置微信小程序的appid
     */
    @Value("${wx.miniapp.config.appid}")
    private String appid;

    /**
     * 设置微信小程序的Secret
     */
    @Value("${wx.miniapp.config.secret}")
    private String secret;

    /**
     * 设置微信小程序消息服务器配置的token
     */
    @Value("${wx.miniapp.config.token:null}")
    private String token;

    /**
     * 设置微信小程序消息服务器配置的EncodingAESKey
     */
    @Value("${wx.miniapp.config.aesKey:null}")
    private String aesKey;

    /**
     * 消息格式，XML或者JSON
     */
    @Value("${wx.miniapp.config.msgDataFormat:null}")
    private String msgDataFormat;

    private static WxMaService maServices;

    public static WxMaService getMaService() {
        return maServices;
    }

    @PostConstruct
    public void init() {
        WxMaDefaultConfigImpl config = new WxMaDefaultConfigImpl();
        config.setAppid(this.appid);
        config.setSecret(this.secret);
        config.setToken(this.token);
        config.setAesKey(this.aesKey);
        config.setMsgDataFormat(this.msgDataFormat);
        maServices = new WxMaServiceImpl();
        maServices.setWxMaConfig(config);
    }
}
