package com.donglan.BYWeApp.controller;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import com.donglan.BYWeApp.common.PageResult;
import com.donglan.BYWeApp.common.Result;
import com.donglan.BYWeApp.config.WxMaConfig;
import com.donglan.BYWeApp.entity.IntegralDetails;
import com.donglan.BYWeApp.entity.User;
import com.donglan.BYWeApp.entity.WxUserInfoEntity;
import com.donglan.BYWeApp.exceptions.BaseException;
import com.donglan.BYWeApp.service.IntegralDetailsService;
import com.donglan.BYWeApp.service.UserService;
import com.donglan.BYWeApp.service.WxMaService;
import me.chanjar.weixin.common.error.WxErrorException;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author shj
 * @date 2021/5/11 11:19 上午
 */
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private IntegralDetailsService integralDetailsService;

    @Autowired
    private HttpServletResponse response;

    @RequestMapping(value = "/search/{page}/{size}",method = RequestMethod.POST)
    @CrossOrigin
    private Result search(@PathVariable Integer page,@PathVariable Integer size,@RequestBody User user){
        Page<User> pageInfo = userService.search(page,size,user);
        return Result.success("查询成功",new PageResult<>(pageInfo.getTotalElements(),pageInfo.getContent()));

    }

    @RequestMapping(value = "/register/{userId}",method = RequestMethod.PUT)
    public Result registerVip(@PathVariable Long userId, @RequestBody WxUserInfoEntity wxUserInfoEntity) throws WxErrorException, IOException {
//        wxMaService.getUserInfo();
        User user = userService.registerVip(userId, wxUserInfoEntity);
        Map<String,Object> ret = CollUtil.newHashMap();
        ret.put("userId",user.getId().toString());
        ret.put("isActive",user.getIsActive());
        return Result.success(ret);
    }

    @RequestMapping(value = "/integral/{userId}")
    public Result findIntegral(@PathVariable Long userId) throws WxErrorException, IOException {
        User user = userService.findUserById(userId);
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        Map<String,Object> ret = CollUtil.newHashMap();
        ret.put("integral",user.getIntegral());
        return Result.success(ret);
    }

    @RequestMapping(value = "/integralDetails/{userId}/{page}/{size}")
    public Result findIntegral(@PathVariable Long userId,@PathVariable Integer page,@PathVariable Integer size) throws WxErrorException, IOException {
        User user = userService.findUserById(userId);
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        IntegralDetails integralDetails = new IntegralDetails();
        integralDetails.setUserId(userId);
        Page<IntegralDetails> pageInfo = integralDetailsService.searchByUserId(page, size, integralDetails);
        List<IntegralDetails> list = pageInfo.getContent();
        list = list.stream().map(item->{
            User tmp =new User();
            tmp.setWxName(item.getUser().getWxName());
            item.setUser(tmp);
            return item;
        }).collect(Collectors.toList());
//        Map<String,Object> ret = CollUtil.newHashMap();
//        ret.put("integral",user.getIntegral());
        return Result.success(new PageResult<>(pageInfo.getTotalPages(),list,pageInfo.getTotalPages()));
    }

    @RequestMapping(value = "/payQRCode/{userId}",method = RequestMethod.GET)
    public void getPayQrCode(@PathVariable Long userId) throws IOException {
        User user = userService.findUserById(userId);
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        String path = user.getPayQrCodePath();
        File file = new File(path);
        InputStream inputStream = FileUtil.getInputStream(file);
        OutputStream outputStream = response.getOutputStream();
        IoUtil.copy(inputStream,outputStream);
        IoUtil.close(inputStream);
        IoUtil.close(outputStream);
    }

    @RequestMapping(value = "/shareWeAppCode/{userId}",method = RequestMethod.GET)
    public void getShareWeAppCode(@PathVariable Long userId) throws IOException {
        User user = userService.findUserById(userId);
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        String path = user.getShareQrCodePath();
        File file = new File(path);
        InputStream inputStream = FileUtil.getInputStream(file);
        OutputStream outputStream = response.getOutputStream();
        IoUtil.copy(inputStream,outputStream);
        IoUtil.close(inputStream);
        IoUtil.close(outputStream);
    }

    @RequestMapping(value="/hasOrderFriends/{userId}/{page}/{size}",method = RequestMethod.GET)
    public Result getHasOrderFriends(@PathVariable Long userId,@PathVariable Integer page,@PathVariable Integer size){
        Page<User> hasOrderFriends =  userService.findHasOrderFriendsByUserIdAndHasOrder(userId,page,size);
        List<User> list = hasOrderFriends.getContent().stream().map(item->{
            User tmp = new User();
            tmp.setId(item.getId());
            tmp.setWxName(item.getWxName());
            tmp.setWxAvatar(item.getWxAvatar());
            return tmp;
        }).collect(Collectors.toList());
        return Result.success("查询成功",new PageResult<>(hasOrderFriends.getTotalElements(),list,hasOrderFriends.getTotalPages()));
    }

    @RequestMapping(value="/noOrderFriends/{userId}/{page}/{size}",method = RequestMethod.GET)
    public Result getNoOrderFriends(@PathVariable Long userId,@PathVariable Integer page,@PathVariable Integer size){
        Page<User> noOrderFriends =  userService.findNoOrderFriendsByUserIdAndHasOrder(userId,page,size);
        List<User> list = noOrderFriends.getContent().stream().map(item->{
            User tmp = new User();
            tmp.setId(item.getId());
            tmp.setWxName(item.getWxName());
            tmp.setWxAvatar(item.getWxAvatar());
            return tmp;
        }).collect(Collectors.toList());
        return Result.success("查询成功",new PageResult<>(noOrderFriends.getTotalElements(),list,noOrderFriends.getTotalPages()));
    }
}
