package com.donglan.BYWeApp.interceptors;

import cn.hutool.core.collection.CollUtil;
import com.donglan.BYWeApp.common.JwtUtils;
import com.donglan.BYWeApp.entity.Admin;
import com.donglan.BYWeApp.exceptions.BaseException;
import com.donglan.BYWeApp.service.AdminService;
import io.jsonwebtoken.Claims;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.file.PathMatcher;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @author shj
 * @date 2021/5/31 2:01 下午
 */
@Component
public class AuthInterceptor extends HandlerInterceptorAdapter {

    @Autowired
    private AdminService adminService;

    List<List<String>> workPermissions;

    public AuthInterceptor() {
       workPermissions = CollUtil.newArrayList();
        workPermissions.add(CollUtil.newArrayList("/order","POST"));
        workPermissions.add(CollUtil.newArrayList("/reserveOrder/waitConsume/**","GET"));
        workPermissions.add(CollUtil.newArrayList("/admin/login","POST"));
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if(request.getMethod().equals("OPTIONS"))
            return true;

        String token = request.getHeader("token");
        Claims claims = JwtUtils.verifyJwt(token);
        if(claims==null) {
            response.addHeader("Access-Control-Allow-Origin","*");
            throw new BaseException(200201, "无效token");
        }
        Long adminId = (Long) claims.get("adminId");
        request.getSession().setAttribute("adminId",adminId);
        String role = (String)claims.get("role");
        if("worker".equals(role)){
            String uri = request.getRequestURI();
            String method = request.getMethod();
            for(List<String> permission : workPermissions){
                String uriPattern = permission.get(0);
                String methodPattern = permission.get(1);
                AntPathMatcher matcher = new AntPathMatcher();
                boolean isUriMatch = matcher.match(uriPattern,uri);
                boolean isMethodMatch = method.equals(methodPattern);
                if(isUriMatch && isMethodMatch) {
                    return true;
                }
            }
            response.addHeader("Access-Control-Allow-Origin","*");
            throw new BaseException(200105, "管理员权限不足");
        }
        return true;
    }


}
