package com.donglan.BYWeApp.service;

import cn.hutool.core.lang.Snowflake;
import com.donglan.BYWeApp.dao.ActivityDao;
import com.donglan.BYWeApp.entity.*;
import com.donglan.BYWeApp.exceptions.BaseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.Date;
import java.util.List;

/**
 * @author shj
 * @date 2021/6/10 2:22 下午
 */
@Service
public class ActivityService {

    @Autowired
    private ActivityDao activityDao;

    @Autowired
    private DictService dictService;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private UserService userService;

    @Autowired
    private OrderService orderService;

    public Activity add(Activity activity) {
        String type = activity.getType();
        Dict dict = dictService.findByTypeAndCode("ACTIVITY_TYPE",type);
        if(dict==null){
             throw new BaseException(200301,"字典数据不存在");
        }
        Long id = snowflake.nextId();
        activity.setId(id);
        activity = activityDao.save(activity);
        return activity;
    }

    public Activity update(Activity activity){
        String type = activity.getType();
        Dict dict = dictService.findByTypeAndCode("ACTIVITY_TYPE",type);
        if(dict==null){
            throw new BaseException(200301,"字典数据不存在");
        }
        return activityDao.save(activity);
    }

    public Page<Activity> search(Integer page, Integer size, Activity activity) {
        Specification<Activity> specification = (Specification<Activity>) (root, cq, cb) -> {
            Predicate predicate = cb.conjunction();
            predicate.getExpressions().add(cb.equal(root.get("deleted"),0));
            return predicate;
        };
        Pageable pageable = PageRequest.of(page - 1 ,size);
        return activityDao.findAll(specification,pageable);
    }

    public void runShareActivity(Order order){
        Date now = new Date();
        List<Activity> activities = activityDao.findAllByTypeAndStartTimeBeforeAndEndTimeAfterAndState("001",now,now,true);
        User user = userService.findUserById(order.getUserId());
        if(user.getReferrerId()!=null){
            if(orderService.findFirstByUserId(user.getId())==null) {
                for (int i = 0;i<activities.size();i++){
                    Activity activity = activities.get(i);
                    userService.addIntegral(user.getReferrerId(), order.getId(),activity.getIntegral() , "邀请验车",activity.getId());
                }
            }
        }
    }

    public void logicDeleteById(Long activityId) {
        Activity activity = activityDao.findById(activityId).get();
        activity.setDeleted(true);
        activityDao.save(activity);
    }
}
