package com.donglan.BYWeApp.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Snowflake;
import com.donglan.BYWeApp.common.JwtUtils;
import com.donglan.BYWeApp.dao.AdminDao;
import com.donglan.BYWeApp.entity.Activity;
import com.donglan.BYWeApp.entity.Admin;
import com.donglan.BYWeApp.exceptions.BaseException;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * @author shj
 * @date 2021/5/31 1:10 下午
 */
@Service
@Transactional
public class AdminService {

    @Autowired
    private AdminDao adminDao;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private HttpServletRequest request;

    public Admin add(Admin admin) {
        if(adminDao.findAdminByUsernameAndState(admin.getUsername(),false)!=null)
            throw new BaseException(200101,"用户名已经被使用");
        Long id = snowflake.nextId();
        admin.setId(id);
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        String password = encoder.encode("cljc123456");
        admin.setPassword(password);
        adminDao.save(admin);
        return admin;
    }

    public String login(Admin admin) {
        Admin dbAdmin =adminDao.findAdminByUsernameAndState(admin.getUsername(),false);
        if(dbAdmin==null)
            throw new BaseException(200102,"用户名错误");
        BCryptPasswordEncoder encoder =new BCryptPasswordEncoder();
        boolean matches = encoder.matches(admin.getPassword(), dbAdmin.getPassword());
        if(!matches){
            throw new BaseException(200103,"密码错误");
        }
        Map<String,Object> params = CollUtil.newHashMap();
        if("admin".equals(admin.getUsername())){
            params.put("role","admin");
        }else{
            params.put("role","worker");
        }
        return JwtUtils.generateAdminToken(dbAdmin.getId(),params);
    }

    public Admin findById(Long adminId) {
        Admin admin = adminDao.findById(adminId).get();
        if(admin == null) {
            throw new BaseException(200104, "管理员不存在");
        }
        return  admin;
    }

    public Page<Admin> search(Integer page, Integer size, Admin admin) {
        Pageable pageable = PageRequest.of(page-1,size);
        return adminDao.findAllByDeleted(false,pageable);
    }

    public void logicDeleteById(Long adminId) {
        Admin admin = adminDao.findById(adminId).get();
        if("admin".equals(admin.getUsername())){
            throw new BaseException(200105,"超级管理员无法删除");
        }
        if(admin.getId() == request.getSession().getAttribute("adminId")){
            throw new BaseException(200106,"不能删除当前账号");
        }
        admin.setDeleted(true);
        adminDao.save(admin);
    }
}
