package com.donglan.BYWeApp.service;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.StrUtil;
import com.donglan.BYWeApp.common.Result;
import com.donglan.BYWeApp.dao.DictDao;
import com.donglan.BYWeApp.dao.DictTypeDao;
import com.donglan.BYWeApp.entity.Dict;
import com.donglan.BYWeApp.entity.DictType;
import com.donglan.BYWeApp.exceptions.BaseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.querydsl.binding.QuerydslPredicate;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.List;

/**
 * @author shj
 * @date 2021/6/9 9:28 上午
 */
@Service
public class DictService {

    @Autowired
    private DictDao dictDao;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private DictTypeService dictTypeService;


    public Dict findById(Long id) {
        return dictDao.findById(id).get();
    }

    public Dict add(Dict dict) {
        String type = dict.getType();
        DictType dictType = dictTypeService.findByType(type);
        if(dictType == null) {
            throw new BaseException(200301, "字典类型不存在");
        }
        Dict dbDict = dictDao.findByTypeAndCode(type, dict.getCode());
        if(dbDict != null) {
            throw new BaseException(200401, "该键值已被使用");
        }
        Long id = snowflake.nextId();
        dict.setId(id);
        return dictDao.save(dict);
    }

    public Dict update(Dict dict) {
        String type = dict.getType();
        DictType dictType = dictTypeService.findByType(type);
        if(dictType == null) {
            throw new BaseException(200301, "字典类型不存在");
        }
        Dict dbDict = dictDao.findByTypeAndCode(type, dict.getCode());
        if(dbDict != null) {
            throw new BaseException(200401, "该键值已被使用");
        }
        return dictDao.save(dict);
    }

    public void deleteById(Long id) {
         dictDao.deleteById(id);
    }

    public Page<Dict> search(Integer page, Integer size, Dict dict){
        Pageable pageable = PageRequest.of(page-1,size);
        Specification<Dict> specification = new Specification<Dict>() {
            @Override
            public Predicate toPredicate(Root<Dict> root, CriteriaQuery<?> cq, CriteriaBuilder cb) {
                Predicate predicate = cb.conjunction();
                if(StrUtil.isNotEmpty(dict.getType())) {
                    predicate.getExpressions().add(cb.equal(root.get("type"), dict.getType()));
                }
//                predicate = cb.and(cb.equal(root.get("type"),dict.getType()));
                return predicate;
            }
        };
        return dictDao.findAll(specification,pageable);
    }

    public List<Dict> findByType(String dataType) {
        DictType dictType = dictTypeService.findByType(dataType);
        if(dictType == null)
            throw new BaseException(200301,"字典类型不存在");
        return dictDao.findAllByType(dataType);
    }

    public Dict findByTypeAndCode(String type,String code){
        return dictDao.findByTypeAndCode(type,code);
    }
}
