package com.donglan.BYWeApp.service;

import cn.hutool.core.lang.Snowflake;
import com.donglan.BYWeApp.dao.DictTypeDao;
import com.donglan.BYWeApp.entity.Dict;
import com.donglan.BYWeApp.entity.DictType;
import com.donglan.BYWeApp.exceptions.BaseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

/**
 * @author shj
 * @date 2021/6/9 9:28 上午
 */
@Service
public class DictTypeService {

    @Autowired
    private DictTypeDao dictTypeDao;

    @Autowired
    private Snowflake snowflake;

    public DictType findByType(String type){
        return dictTypeDao.findByType(type);
    }


    public DictType findById(Long id) {
        return dictTypeDao.findById(id).get();
    }

    public DictType add(DictType dictType) {
        DictType tmp = dictTypeDao.findByType(dictType.getType());
        if(tmp != null)
            throw new BaseException(200302,"字典类型已存在");
        Long id = snowflake.nextId();
        dictType.setId(id);
        return dictTypeDao.save(dictType);
    }

    public DictType update(DictType dictType) {
        DictType dbDictType = dictTypeDao.findById(dictType.getId()).get();
        if(!dbDictType.getType().equals(dictType.getType())) {
            DictType tmp = dictTypeDao.findByType(dictType.getType());
            if (tmp != null)
                throw new BaseException(200302, "字典类型已经存在");
        }
        return dictTypeDao.save(dictType);
    }

    public void deleteById(Long id) {
        dictTypeDao.deleteById(id);
    }

    public Page<DictType> search(Integer page, Integer size, DictType dictType){
        Pageable pageable = PageRequest.of(page-1,size);
        Specification<DictType> specification = new Specification<DictType>() {
            @Override
            public Predicate toPredicate(Root<DictType> root, CriteriaQuery<?> cq, CriteriaBuilder cb) {
                Predicate predicate = cb.conjunction();
                return predicate;
            }
        };
        return dictTypeDao.findAll(specification,pageable);
    }

}
