package com.donglan.BYWeApp.service;

import cn.hutool.core.lang.Snowflake;
import com.donglan.BYWeApp.common.Result;
import com.donglan.BYWeApp.dao.ReserveOrderDao;
import com.donglan.BYWeApp.entity.ReserveOrder;
import com.donglan.BYWeApp.entity.User;
import com.donglan.BYWeApp.exceptions.BaseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.Calendar;
import java.util.Date;

/**
 * @author shj
 * @date 2021/6/9 1:55 下午
 */
@Service
public class ReserveOrderService {
    @Autowired
    private ReserveOrderDao reserveOrderDao;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private UserService userService;

    public ReserveOrder findById(Long id){
        return reserveOrderDao.findById(id).get();
    }

    @Transactional
    public ReserveOrder add(ReserveOrder reserveOrder) {
        User user = userService.findUserById(reserveOrder.getUserId());
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        Date now = new Date();
        ReserveOrder dbReserveOrder = reserveOrderDao.findFirstByCarPlateAndStateAndReserveDateBefore(reserveOrder.getCarPlate(), 1,now);
        if(dbReserveOrder != null){
            reserveOrder.setState(0);
            reserveOrderDao.save(reserveOrder);
        }
        reserveOrder.setId(snowflake.nextId());
        return reserveOrderDao.save(reserveOrder);
    }

    public Page<ReserveOrder> findByUserId(Long userId, Integer page, Integer size) {
        Sort sort = new Sort(Sort.Direction.DESC, "createTime");
        Pageable pageable = PageRequest.of(page-1,size,sort);
        return reserveOrderDao.findByUserId(userId,pageable);
    }

    public Page<ReserveOrder> findUserWaitConsumeList(Long userId, Integer page, Integer size) {
        Sort sort = new Sort(Sort.Direction.DESC, "createTime");
        Pageable pageable = PageRequest.of(page-1,size,sort);
        Date now = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(now);
        calendar.set(Calendar.HOUR_OF_DAY,0);
        calendar.set(Calendar.MINUTE,0);
        calendar.set(Calendar.SECOND,0);
        now = calendar.getTime();
        return reserveOrderDao.findByUserIdAndReserveDateGreaterThanEqualAndState(userId,now,1,pageable);
    }

    public Page<ReserveOrder> search(Integer page, Integer size, ReserveOrder reserveOrder) {
        Specification<ReserveOrder> specification = new Specification<ReserveOrder>() {
            @Override
            public Predicate toPredicate(Root<ReserveOrder> root, CriteriaQuery<?> cq, CriteriaBuilder cb) {
                Predicate predicate = cb.conjunction();
                if(reserveOrder.getState()!=null){
                    predicate.getExpressions().add(cb.and(cb.equal(root.get("state"),reserveOrder.getState())));
                }
                return predicate;
            }
        };
        Sort sort = new Sort(Sort.Direction.DESC,"createTime");
        Pageable pageable = PageRequest.of(page-1,size,sort);
        return reserveOrderDao.findAll(specification,pageable);
    }

    public ReserveOrder update(ReserveOrder reserveOrder) {
        return reserveOrderDao.save(reserveOrder);
    }
}
