package com.donglan.BYWeApp.service;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.extra.qrcode.QrCodeUtil;
import com.donglan.BYWeApp.dao.UserDao;
import com.donglan.BYWeApp.entity.IntegralDetails;
import com.donglan.BYWeApp.entity.User;
import com.donglan.BYWeApp.entity.WxUserInfoEntity;
import com.donglan.BYWeApp.exceptions.BaseException;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;

/**
 * @author shj
 * @date 2021/5/11 11:19 上午
 */
@Service
@Transactional
public class UserService {
    @Autowired
    private UserDao userDao;

    @Autowired
    private WxMaService wxMaService;

    @Autowired
    private Snowflake snowflake;

    @Autowired
    private IntegralDetailsService integralDetailsService;

    public User findUserByOpenId(String openId){
        return userDao.findUserByWxOpenId(openId);
    }

    public User save(User user){
        user = userDao.save(user);
        return  user;
    }

    public User findUserById(Long userId) {
        return userDao.findById(userId).get();
    }

    public User login(WxMaJscode2SessionResult session) {
        String openId = session.getOpenid();
        User user = findUserByOpenId(openId);
        if(user == null){
            user = new User();
            user.setId(snowflake.nextId());
            user.setWxOpenId(session.getOpenid());
            user.setWxUnionId(session.getUnionid());
            user.setSessionKey(session.getSessionKey());
        }else{
            user.setSessionKey(session.getSessionKey());
        }
        userDao.save(user);
        return user;
    }

    public File generateUserShareQRCode(User user) throws WxErrorException, FileNotFoundException {
        File file = wxMaService.generateQrCode(user.getId().toString(), "pages/login/index");
        File dest = new File("shareWeAppCode/"+user.getId().toString()+".jpg");
        FileUtil.touch(dest);
        InputStream is = new FileInputStream(file);
        OutputStream os = new FileOutputStream(dest);
        IoUtil.copy(is,os);
        IoUtil.close(is);
        IoUtil.close(os);
        return dest;
    }

    public File generateUserPayQRCode(User user){
        File file = FileUtil.touch(new File("payQRCodes/" + user.getId().toString() + ".jpg"));
        QrCodeUtil.generate(user.getId().toString(), 300, 300, file);
        return file;
    }

    public void addIntegral(Long userId,Long orderId,Integer integral,String reason){
        User user = findUserById(userId);
        if(user ==null)
            throw  new BaseException(200001,"用户不存在");
        user.setIntegral(user.getIntegral()+integral);
        userDao.save(user);
        IntegralDetails integralDetails = new IntegralDetails();
        integralDetails.setUserId(userId);
        integralDetails.setReason(reason);
        integralDetails.setOrderId(orderId);
        integralDetails.setIntegral(integral);
        integralDetailsService.add(integralDetails);
    }

    public void addIntegral(Long userId,Long orderId,Integer integral,String reason,Long activityId){
        User user = findUserById(userId);
        if(user ==null)
            throw  new BaseException(200001,"用户不存在");
        user.setIntegral(user.getIntegral()+integral);
        userDao.save(user);
        IntegralDetails integralDetails = new IntegralDetails();
        integralDetails.setUserId(userId);
        integralDetails.setReason(reason);
        integralDetails.setOrderId(orderId);
        integralDetails.setIntegral(integral);
        integralDetails.setActivityId(activityId);
        integralDetailsService.add(integralDetails);
    }

    public User  registerVip(Long userId, WxUserInfoEntity wxUserInfoEntity) throws WxErrorException, FileNotFoundException {
        User user = findUserById(userId);
        if(user==null)
            throw new BaseException(200001,"用户不存在");
        if(user.getIsActive())
            throw new BaseException(200002,"用户已经是会员，不可重复开通");
        WxMaUserInfo wxUserInfo = wxMaService.getUserInfo(user.getSessionKey(), wxUserInfoEntity.getSignature(), wxUserInfoEntity.getRawData(),
                wxUserInfoEntity.getEncryptedData(), wxUserInfoEntity.getIv());
        user.setWxName(wxUserInfo.getNickName());
        user.setWxAvatar(wxUserInfo.getAvatarUrl());
        user.setIsActive(true);
        user.setReferrerId(wxUserInfoEntity.getReferrerId());
        File payQRCode = generateUserPayQRCode(user);
        FileUtil.touch(payQRCode);
        user.setPayQrCodePath(payQRCode.getPath());
        File file = generateUserShareQRCode(user);
        user.setShareQrCodePath(file.getPath());
        return save(user);
    }

    public Page<User> findHasOrderFriendsByUserIdAndHasOrder(Long referrerId, Integer page, Integer size) {
        Pageable pageable = PageRequest.of(page-1,size);
        Page<User> friends = userDao.findHasOrderUserByReferrerId(referrerId, pageable);
        return friends;
    }

    public Page<User> findNoOrderFriendsByUserIdAndHasOrder(Long referrerId, Integer page, Integer size) {
        Pageable pageable = PageRequest.of(page-1,size);
        Page<User> friends = userDao.findNoOrderUserByReferrerId(referrerId, pageable);
        return friends;
    }

    public Page<User> search(Integer page, Integer size, User user) {
        Pageable pageable = PageRequest.of(page-1,size);
        return userDao.findAll(pageable);
    }
}
